library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity taktgenerator is
generic (teiler : positive := 50000000);
port (
		clock					: in	std_logic;  -- Systemclock Pin
		sekunden_ziffer0  : out std_logic_vector(3 downto 0);
		sekunden_ziffer1  : out std_logic_vector(3 downto 0);
		minuten_ziffer0   : out std_logic_vector(3 downto 0);
		minuten_ziffer1   : out std_logic_vector(3 downto 0);
		stunden_ziffer0   : out std_logic_vector(3 downto 0);
		stunden_ziffer1   : out std_logic_vector(3 downto 0)
		);
		
end taktgenerator;

architecture taktgenerator_rtl of taktgenerator is
	subtype ziffer is unsigned(3 downto 0);
   type doppel_ziffer is array(1 downto 0) of ziffer;
	signal sekunden_neu, minuten_neu, stunden_neu : doppel_ziffer := (others => (others => '0'));
   signal sekunden_alt, minuten_alt : doppel_ziffer := (others => (others => '0'));

	signal zaehler : integer range 0 to teiler;
begin
	 -- Teilt die Taktfrequenz von 50 MHz durch 50.000.000 = 1 Hz
	 process (clock)
	 begin
		if rising_edge(clock)
		then
			if zaehler < (teiler-1)
				then zaehler <= zaehler + 1;
				else zaehler <= 0;
			end if;
      end if;
    end process;
   
	 -- Ziffernweises Increment der Sekunden
	 process (clock)
	 begin
		if rising_edge(clock)
		then
			if zaehler = 0
			then				
				if sekunden_neu(0) < 9
				then
					sekunden_neu(0) <= sekunden_neu(0) + 1;
				else 
					sekunden_neu(0) <= (others => '0');
					if sekunden_neu(1) < 5
					then
						sekunden_neu(1) <= sekunden_neu(1) + 1;
					else
						sekunden_neu(1) <= (others => '0');
					end if;
				end if;
				-- Fr Erkennung des berganges 59->00 bentigt
				-- Verhindert beim Start mit 00 Sekunden den bertrag auf Minuten
				sekunden_alt(0) <= sekunden_neu(0);
				sekunden_alt(1) <= sekunden_neu(1);
			end if;
		end if;
	 end process;
	 
	 -- Ziffernweises Inkrement der Minuten
	 process (clock)
	 begin
		if rising_edge(clock)
		then
			if (sekunden_alt(1) = 5 and sekunden_alt(0) = 9) and (sekunden_neu(1) = 0 and sekunden_neu(0) = 0)
			then
				if minuten_neu(0) < 9
				then
					minuten_neu(0) <= minuten_neu(0) + 1;
				else
					minuten_neu(0) <= (others => '0');
					if minuten_neu(1) < 5
					then
						minuten_neu(1) <= minuten_neu(1) + 1;
					else
						minuten_neu(1) <= (others => '0');
					end if;
				end if;
			end if;
			-- Fr Erkennung des berganges 59->00 bentigt
			-- Verhindert beim Start mit 00 Minuten den bertrag auf Stunden
			minuten_alt(0) <= minuten_neu(0);
			minuten_alt(1) <= minuten_neu(1);
		end if;
	 end process;
	
	 -- Ziffernweises Inkrement der Stunden
	 process (clock)
	 begin
		if rising_edge(clock)
		then
			if (minuten_alt(1) = 5 and minuten_alt(0) = 9) and (minuten_neu(1) = 0 and minuten_neu(0) = 0)
			then
				if stunden_neu(0) < 3
				then
					stunden_neu(0) <= stunden_neu(0) + 1;
				else
					stunden_neu(0) <= (others => '0');
					if stunden_neu(1) < 2
					then
						stunden_neu(1) <= stunden_neu(1) + 1;
					else
						stunden_neu(1) <= (others => '0');
					end if;
				end if;
			end if;
		end if;
	 end process;
	 
	 -- Ziffernweise Ausgabe an die Anzeige
	 sekunden_ziffer0 <= std_logic_vector(sekunden_neu(0));
	 sekunden_ziffer1 <= std_logic_vector(sekunden_neu(1));
	 minuten_ziffer0	<= std_logic_vector(minuten_neu(0));
	 minuten_ziffer1  <= std_logic_vector(minuten_neu(1));
	 stunden_ziffer0  <= std_logic_vector(stunden_neu(0));
	 stunden_ziffer1  <= std_logic_vector(stunden_neu(1));
end taktgenerator_rtl;


			
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
			
entity anzeige is 			
port (
		clock    	: in std_logic;		-- Systemclock Pin		
		lcd_data 	: out std_logic_vector(7 downto 0);
		lcd_enable  : out std_logic;
		lcd_rs   	: out std_logic;
		lcd_rw   	: out std_logic;
		led         : out std_logic_vector(7 downto 0)
		);
		
end anzeige;

architecture anzeige_rtl of anzeige is

	component taktgenerator is
	port
	(		
		clock					: in	std_logic;  -- Systemclock Pin
		sekunden_ziffer0  : out std_logic_vector(3 downto 0);
		sekunden_ziffer1  : out std_logic_vector(3 downto 0);
		minuten_ziffer0   : out std_logic_vector(3 downto 0);
		minuten_ziffer1   : out std_logic_vector(3 downto 0);
		stunden_ziffer0   : out std_logic_vector(3 downto 0);
		stunden_ziffer1   : out std_logic_vector(3 downto 0)
	);
	end component taktgenerator;
	
	subtype ziffer is std_logic_vector(3 downto 0);
   type ziffern is array(5 downto 0) of ziffer;
	signal lcd_anzeige : ziffern;
	type states is (one, one_two, two, three, four, five, six, seven, eigth, nine, ten,
						 eleven, twelf, thirteen, fourteen, fifteen, sixteen, seventeen,
						 eightteen, nineteen, twenty, twenty_one, twenty_two, twenty_three);
	signal state : states;
	-- Power-On Delay
	signal delay : integer range 750000 downto 0 := 750000;
begin

	takt : taktgenerator
	port map
	(
		clock   				=> clock,  -- clock Entity clock = Systemclock
		sekunden_ziffer0 	=> lcd_anzeige(0),
		sekunden_ziffer1 	=> lcd_anzeige(1),
		minuten_ziffer0 	=> lcd_anzeige(2),
		minuten_ziffer1 	=> lcd_anzeige(3),
		stunden_ziffer0 	=> lcd_anzeige(4),
		stunden_ziffer1 	=> lcd_anzeige(5)
	);
	
	process (clock)
	begin
		if rising_edge(clock)
		then
			-- Nutze die Sekunden zur Ansteuerung der LEDs
			led(3 downto 0) <= lcd_anzeige(0);
			led(7 downto 4) <= lcd_anzeige(1);
		end if;
	end process;
	
	process (clock)
	begin
		if rising_edge(clock)
		then
		   case state is
			------------------------- Power-On Initialisierung ------------------------------
			-- Siehe User Guide, Seite 54, Power-On Initialisierung
			---------------------------------------------------------------------------------
				when one => 		-- Warte mindestens 15 ms nach dem Einschalten des Boards
										-- 750.000 Taktzyklen bei 50 MHz
										lcd_enable <= '0';
										lcd_rs <= '0';
										lcd_rw <= '0';
									
										if delay > 0
										then
											delay <= delay - 1;
										else
											state <= one_two;
										end if;
				when one_two =>   lcd_enable <= '1';
										-- Schalte in den 8 Bit Modus
										lcd_data <= "00110011";
										delay <= 12;
										state <= two;
				when two =>			-- Warte mindestens 230 ns = 12 Taktzyklen bei 50 MHz
										lcd_data(7 downto 4) <= "0011";
										if delay > 0
										then
											delay <= delay - 1;
										else
											delay <= 205000;
											state <= three;
										end if;
				when three =>      -- Warte minestens 4,1 ms = 205.000 Taktzyklen
										if delay > 0
										then
											delay <= delay - 1;
										else
											delay <= 12;
											state <= four;
										end if;
				when four =>      -- Warte mindestens 230 ns = 12 Taktzyklen bei 50 MHz
										lcd_data(7 downto 4) <= "0011";
										if delay > 0
										then
											delay <= delay - 1;
										else
											delay <= 5000;
											state <= five;
										end if;
				when five =>		-- Warte mindestens 100 us = 5000 Taktzyklen bei 50 MHz
										if delay > 0
										then
											delay <= delay - 1;
										else
											delay <= 12;
											state <= six;
										end if;
				when six => 		-- Warte mindestens 230 ns = 12 Taktzyklen bei 50 MHz
										lcd_data(7 downto 4) <= "0011";
										if delay > 0
										then
											delay <= delay - 1;
										else
											delay <= 2000;
											state <= seven;
										end if;
				when seven =>  	-- Warte mindestens 40 us = 2000 Taktzyklen bei 50 MHz
										if delay > 0
										then
											delay <= delay - 1;
										else
											delay <= 12;
											state <= eigth;
										end if;
				when eigth =>		-- Warte mindestens 230 ns = 12 Taktzyklen bei 50 MHz
										lcd_data(7 downto 4) <= "0010";
										if delay > 0
										then
											delay <= delay - 1;
										else
											delay <= 2000;
											state <= nine;
										end if;
				when nine =>      -- Warte mindestens 40 us = 2.000 Taktzyklen bei 50 MHz
										if delay > 0
										then
											delay <= delay - 1;
										else
											state <= ten;
										end if;
				----------------------- Display Configuration ------------------
				-- Siehe User Guide, Seite 54, Display Configuration
				----------------------------------------------------------------
				when ten =>			-- Function Set Command
										lcd_data <= "00101000";
										state <= eleven;
				when eleven =>		-- Entry Mode Set Command (Increment the address pointer)
										lcd_data <= "00000110";
										state <= twelf;
				when twelf =>		-- Display On (Disable cursor and blinking)
										lcd_data <= "00001100";
										state <= thirteen;
				when thirteen =>  -- Clear Display Command
										lcd_data <= "00000001";
										delay <= 82000;
										state <= fourteen;
				when fourteen =>  if delay > 0
										then
											delay <= delay - 1;
										else
											state <= fifteen;
										end if;
			   ----------------- Von links nach rechts Schreiben ----------------------
				when fifteen =>	-- Set DD RAM Address = 08
										lcd_rs <= '0';
										lcd_rw <= '0';
										lcd_data(7) <= '1';
										lcd_data(6 downto 0) <= "0001000";
										state <= sixteen;
				----------------- Schreibe Stunden --------------------------
				when sixteen =>	-- Write Data to DD RAM
										lcd_rs <= '1';
										lcd_rw <= '0';
										lcd_data(7 downto 4) <= "0011";
										lcd_data(3 downto 0) <= lcd_anzeige(5);
										state <= seventeen;
				when seventeen =>	-- Write Data to DD RAM
										lcd_rs <= '1';
										lcd_rw <= '0';
										lcd_data(7 downto 4) <= "0011";
										lcd_data(3 downto 0) <= lcd_anzeige(4);												
										state <= eightteen;
				----------------- Schreibe Doppelpunkt ---------------------
				when eightteen => -- Write Data to DD RAM
										lcd_rs <= '1';
										lcd_rw <= '0';
										lcd_data(7 downto 4) <= "0011";
										lcd_data(3 downto 0) <= "1010";												
										state <= nineteen;
				----------------- Schreibe Minuten -------------------------
				when nineteen =>  -- Write Data to DD RAM
										lcd_rs <= '1';
										lcd_rw <= '0';
										lcd_data(7 downto 4) <= "0011";
										lcd_data(3 downto 0) <= lcd_anzeige(3);												
										state <= twenty;
				when twenty =>    -- Write Data to DD RAM
										lcd_rs <= '1';
										lcd_rw <= '0';
										lcd_data(7 downto 4) <= "0011";
										lcd_data(3 downto 0) <= lcd_anzeige(2);												
										state <= twenty_one;
				----------------- Schreibe Doppelpunkt -------------------						
				when twenty_one =>-- Write Data to DD RAM
										lcd_rs <= '1';
										lcd_rw <= '0';
										lcd_data(7 downto 4) <= "0011";
										lcd_data(3 downto 0) <= "1010";												
										state <= twenty_two;						
				----------------- Schreibe Sekunden ----------------------
				when twenty_two =>-- Write Data to DD RAM
										lcd_rs <= '1';
										lcd_rw <= '0';
										lcd_data(7 downto 4) <= "0011";
										lcd_data(3 downto 0) <= lcd_anzeige(1);												
										state <= twenty_three;
				when twenty_three=>-- Write Data to DD RAM
										lcd_rs <= '1';
										lcd_rw <= '0';
										lcd_data(7 downto 4) <= "0011";
										lcd_data(3 downto 0) <= lcd_anzeige(0);												
			end case;
		end if;
	end process;
	
end anzeige_rtl;